package tech.zapt.zapt_sdk_flutter;

import android.content.Context;

import androidx.annotation.NonNull;
import java.util.Map;
import java.util.Objects;

import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** ZaptSdkFlutterPlugin */
public class ZaptSdkFlutterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

  private MethodChannel channel;
  private ZaptSDKModule zaptSDKModule;
  private BinaryMessenger binaryMessenger;
  public ZaptSdkFlutterPlugin(){}

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    Context applicationContext = flutterPluginBinding.getApplicationContext();
    zaptSDKModule = new ZaptSDKModule(applicationContext);
    binaryMessenger = flutterPluginBinding.getBinaryMessenger();
    channel = new MethodChannel(binaryMessenger, "zapt_sdk_flutter");
    channel.setMethodCallHandler(this);
    flutterPluginBinding.getPlatformViewRegistry().registerViewFactory("zapt_web_view", new ZaptWebViewFactory(binaryMessenger));
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "getMapLink":
        Map<String,String> options = call.argument("options");
        String mapLink = zaptSDKModule.getMapLink(Objects.requireNonNull(call.argument("placeId")), options);
        result.success(mapLink);
        break;
      case "requestPermissions":
        zaptSDKModule.requestPermissions();
        result.success(true);
        break;
      case "initialize":
        String placeId = call.arguments();
        zaptSDKModule.initializeZaptSDK(placeId);
        result.success(true);
        break;
      case "setVisitable":
        String newPlaceId = call.arguments();
        zaptSDKModule.setVisitable(newPlaceId);
        result.success(true);
        break;
      case "requestPermissionsBackground":
        Map<String, String> alertOptions = call.arguments();
        zaptSDKModule.requestPermissionsBackground(alertOptions);
        result.success(true);
        break;
      case "setDisableSyncingForPositioning":
        zaptSDKModule.setDisableSyncingForPositioning(call.arguments());
        result.success(true);
        break;
      case "setDisableSyncingForAnalytics":
        zaptSDKModule.setDisableSyncingForAnalytics(call.arguments());
        result.success(true);
        break;
      case "setDebug":
        zaptSDKModule.setDebug(call.arguments());
        result.success(true);
        break;
      case "addListener":
        zaptSDKModule.addListener(call.arguments(), this.binaryMessenger);
        result.success(true);
        break;
      case "removeLocationListener":
        zaptSDKModule.removeListeners();
        result.success(true);
        break;
      default:
        result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }


  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    zaptSDKModule.setActivity(binding.getActivity());
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    zaptSDKModule.setActivity(binding.getActivity());
  }

  @Override
  public void onDetachedFromActivity() {

  }
}