package tech.zapt.zapt_sdk_flutter;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Build;
import android.text.TextUtils;
import android.view.View;
import android.webkit.WebSettings;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.platform.PlatformView;
import tech.zapt.sdk.webapp.ZaptWebView;

public class ZaptWebViewPlatFormView implements PlatformView, MethodChannel.MethodCallHandler {

    private ZaptWebView webView;
    private BinaryMessenger messenger;

    protected static final String BLANK_URL = "about:blank";

    ZaptWebViewPlatFormView(@NonNull Context context, BinaryMessenger messenger, int id, @Nullable Map<String, Object> creationParams){

        this.messenger = messenger;
        MethodChannel methodChannel = new MethodChannel(messenger, "zapt_web_view_" + id);
        methodChannel.setMethodCallHandler(this);

        initWebView(context, creationParams);
    }

    @Nullable
    @Override
    public View getView() {
        return webView;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        switch (call.method){
            case "loadURL":
                webView.loadUrl((String) call.arguments);
                break;
            case "runJavascript":
                webView.evaluateJavaScriptWithFallback((String) call.arguments);
                break;
            default:
                result.notImplemented();
        }
    }

    @Override
    public void dispose() {}

    void initWebView(Context context, Map<String, Object> params){
        webView = new ZaptWebView(context);
        WebSettings settings = webView.getSettings();
        loadInitialSettingsWebView(settings);

        if(params != null && !params.isEmpty()){

            if(params.containsKey("initialURL")){
                String initialURL = (String) params.get("initialURL");
                if(!TextUtils.isEmpty(initialURL)){
                    webView.loadUrl(initialURL);
                }else{
                    webView.loadUrl(BLANK_URL);
                }
            }

            if(params.containsKey("injectedJavaScript")){
                String injectedJavaScript = (String) params.get("injectedJavaScript");
                if(!TextUtils.isEmpty(injectedJavaScript)){
                    webView.setInjectedJS(injectedJavaScript);
                }
            }
        }
    }

    @SuppressLint("SetJavaScriptEnabled")
    void loadInitialSettingsWebView(WebSettings settings){
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            settings.setMixedContentMode(0);
            webView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
        } else if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            webView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
        } else {
            webView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
        }

    }
}
