#import "ZaptSdkFlutterPlugin.h"

FlutterEventSink beaconEventSinker = nil;
NSObject<FlutterPluginRegistrar>* flutterPluginRegistrar;

@implementation ZaptSdkFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  flutterPluginRegistrar = registrar;
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"zapt_sdk_flutter"
            binaryMessenger:[registrar messenger]];




  
  ZaptSdkFlutterPlugin* instance = [[ZaptSdkFlutterPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else if ([@"initialize" isEqualToString:call.method]) {
    [self initialize:call.arguments];
    result(@"ok");
  } else if ([@"getMapLink" isEqualToString:call.method]) {
    result([self getMapLink]);
  } else if ([@"requestPermissions" isEqualToString:call.method]) {
    [self requestPermissions];
    result(@"ok");
  } else if ([@"requestPermissionsBackground" isEqualToString:call.method]) {
    [self requestPermissionsBackground];
    result(@"ok");
  } else if ([@"setDisableSyncingForPositioning" isEqualToString:call.method]) {
    [self setDisableSyncingForPositioning:call.arguments];
    result(@"ok");
  } else if ([@"setDisableSyncingForAnalytics" isEqualToString:call.method]) {
    [self setDisableSyncingForAnalytics:call.arguments];
    result(@"ok");
  } else if ([@"addListener" isEqualToString:call.method]) {
    [self addListener:call.arguments];
    result(@"ok");
  }
  
  
  
  else {
    result(FlutterMethodNotImplemented);
  }
}

- (void) initialize:(NSString *)placeID {
    if (!self.isInitialized) {
        self.zaptSDK = [[ZTLocationSDK alloc] initWithVisitableId:placeID];
        self.zaptSDK.disableSyncingForPositioning = true; // positioning is calculated by map
        [self.zaptSDK start];
        [self initLocationManager];
        self.isInitialized = TRUE;
    }
}

- (void) initLocationManager {
    self.locationManager = [[CLLocationManager alloc] init];
    self.locationManager.delegate = self;
    
    CLRegion* region = [[CLBeaconRegion alloc] initWithProximityUUID:[[NSUUID alloc] initWithUUIDString: @"0428C59E-25B7-4E9D-B3CD-D126A46E6E9D"] identifier: @"ZAPT_TECH_FLUTTER"];
    CLRegion* region2 = [[CLBeaconRegion alloc] initWithProximityUUID:[[NSUUID alloc] initWithUUIDString: @"92ed3fd4-f465-11ec-b939-0242ac120002"] identifier: @"ZAPT_TECH_FLUTTER2"];

    NSLog(@"startRangingBeaconsInRegion");
    [self.locationManager startRangingBeaconsInRegion:(CLBeaconRegion*)region];
    NSLog(@"startMonitoringForRegion");
    [self.locationManager startMonitoringForRegion:(CLBeaconRegion*)region];
    [self.locationManager startMonitoringForRegion:(CLBeaconRegion*)region2];
}

- (void)locationManager:(CLLocationManager *)manager didRangeBeacons:(NSArray *)beacons inRegion:(CLBeaconRegion *)region {
    if ([region.identifier  isEqual: @"ZAPT_TECH_FLUTTER"]) {
        if ( [beacons count] > 0) {
            NSMutableArray* beaconsMapsArray = [NSMutableArray new];
            for (CLBeacon* beacon in beacons) {
                NSDictionary* dictOfBeacon = [self mapOfBeacon:beacon];
                [beaconsMapsArray addObject:dictOfBeacon];
            }
            
            [self onBeaconsFound:beaconsMapsArray];
        }
    }
}

-(void)locationManager:(CLLocationManager *)manager didEnterRegion:(CLRegion *)region {
    if ([region.identifier  isEqual: @"ZAPT_TECH_FLUTTER"] || [region.identifier  isEqual: @"ZAPT_TECH_FLUTTER2"]) {
        NSLog(@"didEnterRegion");
        CLRegion* rangeRegion = [[CLBeaconRegion alloc] initWithProximityUUID:[[NSUUID alloc] initWithUUIDString: @"0428C59E-25B7-4E9D-B3CD-D126A46E6E9D"] identifier: @"ZAPT_TECH_FLUTTER"];

        //TODO
        // [self sendEventWithName:@"ReactNativeZaptSdkBeaconsRegionEnter" body:@"ZAPT_TECH_FLUTTER"];
        [self.locationManager startRangingBeaconsInRegion:(CLBeaconRegion*)rangeRegion];
    }
}

-(void)locationManager:(CLLocationManager *)manager didExitRegion:(CLRegion *)region {
    if ([region.identifier  isEqual: @"ZAPT_TECH_FLUTTER"]) {
        NSLog(@"didExitRegion");
        //TODO
        //[self sendEventWithName:@"ReactNativeZaptSdkBeaconsRegionExit" body:@"ZAPT_TECH_FLUTTER"];
    }
}

- (NSDictionary*) mapOfBeacon: (CLBeacon*) beacon {
    NSMutableDictionary* dict = [[NSMutableDictionary alloc] init];
    
    // uuid
    NSString* uuid = beacon.proximityUUID.UUIDString;
    [dict setObject:uuid forKey:@"uuid"];
    
    // major
    [dict setObject:beacon.major forKey:@"major"];
    
    // minor
    [dict setObject:beacon.minor forKey:@"minor"];
    
    // rssi
    NSNumber * rssi = [[NSNumber alloc] initWithInteger:beacon.rssi];
    [dict setObject:rssi forKey:@"rssi"];
    // TODO: Tx value not available from CLBeacon, but possible from CBCentralManager scan on detection
    
    // accuracy is a rough estimate of distance in metres. capped to two decimal places
    NSNumber *accuracy = [NSNumber numberWithDouble:round(100*beacon.accuracy)/100];
    [dict setObject:accuracy forKey:@"accuracy"];
    
    return dict;
}

- (NSString*) getMapLink {
  return [self.zaptSDK getMapLink];
}

- (void) requestPermissions {
  [self.zaptSDK requestPermissions];
}

- (void) requestPermissionsBackground {
  [self.zaptSDK requestPermissionsBackground];
}

- (void) setDisableSyncingForPositioning:(BOOL) disable {
    if (self.isInitialized) {
      self.zaptSDK.disableSyncingForPositioning = disable;
    }
}

- (void) setDisableSyncingForAnalytics:(BOOL) disable {
    if (self.isInitialized) {
      self.zaptSDK.disableSyncingForAnalytics = disable;
    }
}

- (void) addListener:(NSString*) eventName {

  BeaconStreamHandler* beaconStreamHandler =
      [[BeaconStreamHandler alloc] init];
  FlutterEventChannel* eventChannel =
      [FlutterEventChannel eventChannelWithName:eventName
                                binaryMessenger:[flutterPluginRegistrar messenger]];
  [eventChannel setStreamHandler:beaconStreamHandler];
}

- (void) onBeaconsFound:(NSMutableArray*)beacons {
    if(beaconEventSinker != nil) {
      beaconEventSinker(beacons);
    }
}

@end

@implementation BeaconStreamHandler
- (FlutterError*)onListenWithArguments:(id)arguments eventSink:(FlutterEventSink)eventSink {
    beaconEventSinker = eventSink;
  return nil;
}

- (FlutterError*)onCancelWithArguments:(id)arguments {
    beaconEventSinker = nil;
  return nil;
}
@end
