import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'zapt_sdk_flutter_method_channel.dart';

abstract class ZaptSdkFlutterPlatform extends PlatformInterface {
  /// Constructs a ZaptSdkFlutterPlatform.
  ZaptSdkFlutterPlatform() : super(token: _token);

  static final Object _token = Object();

  static ZaptSdkFlutterPlatform _instance = MethodChannelZaptSdkFlutter();

  /// The default instance of [ZaptSdkFlutterPlatform] to use.
  ///
  /// Defaults to [MethodChannelZaptSdkFlutter].
  static ZaptSdkFlutterPlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [ZaptSdkFlutterPlatform] when
  /// they register themselves.
  static set instance(ZaptSdkFlutterPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<String?> getMapLink(Map<String, dynamic> arguments) {
    throw UnimplementedError('getMapLink() has not been implemented.');
  }

  Future<void> requestPermissions() {
    throw UnimplementedError('requestPermissions() has not been implemented');
  }

  Future<void> requestPermissionsBackground(Map<String, String> arguments) {
    throw UnimplementedError(
        'requestPermissionsBackground() has not been implemented');
  }

  Future<void> initialize(String placeId) {
    throw UnimplementedError('initialize() has not been implemented');
  }

  Future<void> setVisitable(String placeId) {
    throw UnimplementedError('setVisitable() has not been implemented');
  }

  Future<void> setDisableSyncingForPositioning(bool disable) {
    throw UnimplementedError(
        'setDisableSyncingForPositioning() has not been implemented');
  }

  Future<void> setDisableSyncingForAnalytics(bool disable) {
    throw UnimplementedError(
        'setDisableSyncingForAnalytics() has not been implemented');
  }

  Future<void> addListener(String eventName) {
    throw UnimplementedError('addListener() has not been implemented');
  }

  Future<void> removeLocationListener() {
    throw UnimplementedError(
        'removeLocationListener() has not been implemented');
  }
}
