import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

typedef ZaptWebViewCreatedCallback = void Function(ZaptWebViewController controller);

const String viewType = 'zapt_web_view';

class ZaptWebView extends StatelessWidget {
  const ZaptWebView({
    super.key,
    this.initialURL,
    this.activeTTS,
    this.injectedJavaScript,
    this.onZaptWebViewCreated,
  });

  final String? initialURL;
  final String? injectedJavaScript;
  final bool? activeTTS;
  final ZaptWebViewCreatedCallback? onZaptWebViewCreated;

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "initialURL": initialURL ?? "",
      "injectedJavaScript": injectedJavaScript ?? "",
      "activeTTS": activeTTS ?? false
    };

    return PlatformViewLink(
      viewType: viewType,
      surfaceFactory: (context, controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (params) {
        return PlatformViewsService.initSurfaceAndroidView(
          id: params.id,
          viewType: viewType,
          layoutDirection: TextDirection.ltr,
          creationParams: creationParams,
          creationParamsCodec: const StandardMessageCodec(),
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(
              (id) => _onPlatformViewCreated(id))
          ..create();
      },
    );
  }

  void _onPlatformViewCreated(int id) {
    if (onZaptWebViewCreated != null) {
      onZaptWebViewCreated!(ZaptWebViewController._(id));
    }
  }
}

class ZaptWebViewController {
  ZaptWebViewController._(int id)
      : _channel = MethodChannel("zapt_web_view_$id");

  final MethodChannel _channel;

  Future<void> loadUrl(String url) {
    return _channel.invokeListMethod('loadURL', url);
  }

  Future<void> runJavascript(String script) {
    return _channel.invokeListMethod("runJavascript", script);
  }
}
